#include <stdio.h>
#include <sys/mmapvxd.h>

struct far_ptr {
  unsigned int offset; 
  unsigned short segment;
};
static struct far_ptr mmapvxd_api = {0, 0};

static int get_vxd_pm_api(unsigned short id, 
			  char *vxdname, 
			  struct far_ptr *api)
{
  asm ("pushw  %%es\n\t"
       "movw   %%ds, %%ax\n\t"
       "movw   %%ax, %%es\n\t"
       "movw   $0x1684, %%ax\n\t"
       "int    $0x2f\n\t"
       "movw   %%es, %%ax\n\t"
       "popw   %%es"
       : "=a" (api->segment), "=D" (api->offset) : "b" (id), "D" (vxdname));
  api->offset &= 0x0000ffff;
  return api->offset != 0;
}

static char mmapvxd_devicename[] = "MMAP    ";

static int mmapvxd_getapi(void)
{
  int res;
  printf ("mmapvxd_getapi called\n");
  res = get_vxd_pm_api(0, mmapvxd_devicename, &mmapvxd_api);
  if (!res) 
    printf ("mmapvxd_getapi failed\n");
  else
    printf ("mmapvxd_getapi succeeded: %04x:%08x\n", 
	    mmapvxd_api.segment,
	    mmapvxd_api.offset);
  return res ? 0 : -1;
}

int mmapvxd_pagereserve(int npages)
{
  int res;
  if (!mmapvxd_api.offset) {
    res = mmapvxd_getapi();
    if (res)
      return res;
  }
  /* pagereserve VxD call returns -1 on error, base address otherwise */
  printf ("pagereserve %d\n", npages);
  asm ("lcall %1"
       : "=a" (res)
       : "m" (mmapvxd_api), "a" (3), "c" (npages));
  printf ("pagereserve res %08x\n", res);
  return res;
}

int mmapvxd_pagecommit(int addr, int size)
{
  int res;
  if (!mmapvxd_api.offset) {
    res = mmapvxd_getapi();
    if (res)
      return res;
  }
  asm ("lcall %1\n\t"
       "xor   %%eax, %%eax\n\t"
       "jnc   1f\n\t"
       "mov   $-1, %%eax\n"
       "1:"
       : "=a" (res)
       : "m" (mmapvxd_api), "a" (4), "c" (size), "d" (addr));
  return res;
}

int mmapvxd_pagedecommit(int addr, int size)
{
  int res;
  if (!mmapvxd_api.offset) {
    res = mmapvxd_getapi();
    if (res)
      return res;
  }
  asm ("lcall %1\n\t"
       "xor   %%eax, %%eax\n\t"
       "jnc   1f\n\t"
       "mov   $-1, %%eax\n"
       "1:"
       : "=a" (res)
       : "m" (mmapvxd_api), "a" (5), "c" (size), "d" (addr));
  return res;
}

int mmapvxd_pagefree(int addr)
{
  int res;
  if (!mmapvxd_api.offset) {
    res = mmapvxd_getapi();
    if (res)
      return res;
  }
  asm ("lcall %1\n\t"
       "xor   %%eax, %%eax\n\t"
       "jnc   1f\n\t"
       "mov   $-1, %%eax\n"
       "1:"
       : "=a" (res)
       : "m" (mmapvxd_api), "a" (6), "d" (addr));
  return res;
}

typedef unsigned long ULONG;

typedef struct _MEMORY_BASIC_INFORMATION {
    ULONG mbi_BaseAddress;
    ULONG mbi_AllocationBase;
    ULONG mbi_AllocationProtect;
    ULONG mbi_RegionSize;
    ULONG mbi_State;
    ULONG mbi_Protect;
    ULONG mbi_Type;
} MEMORY_BASIC_INFORMATION, *PMEMORY_BASIC_INFORMATION;

static MEMORY_BASIC_INFORMATION meminfo;

static int mmapvxd_pagequery(ULONG addr, PMEMORY_BASIC_INFORMATION pmbi)
{
  int res;
  if (!mmapvxd_api.offset) {
    res = mmapvxd_getapi();
    if (res)
      return res;
  }
  asm ("lcall %1\n\t"
       : "=a" (res)
       : "m" (mmapvxd_api), "a" (1), "S" (addr), 
         "D" (pmbi), "c" (sizeof(*pmbi)));
  return res;
}


int mmapvxd_committed(int addr, int size)
{
  int end;
  /* get page-aligned ends for region */
  end = (addr + size + 0xfff) & ~0xfff;
  addr = addr & ~0xfff;
  while (addr < end) {
    if (!mmapvxd_pagequery (addr, &meminfo))
      return 0;  /* error in pagequery */
#if 0
    printf ("mmapvxd_commit: check [%x,%x]; region [%x,%x] state %x\n",
	    addr, end, meminfo.mbi_BaseAddress, 
	    meminfo.mbi_BaseAddress + meminfo.mbi_RegionSize,
	    meminfo.mbi_State);
#endif
    if (meminfo.mbi_State != 0x1000)
      return 0;  /* not committed */
    addr = meminfo.mbi_BaseAddress + meminfo.mbi_RegionSize;
  }
  return 1;
}




