#include <stdio.h>
#include <stddef.h>
#include <string.h>
#include <io.h>
#include <dpmi.h>

struct far_ptr {
  unsigned int offset; 
  unsigned short segment;
} api = {0,0};

int get_vxd_pm_api(unsigned short id, char *vxdname, struct far_ptr *api)
{
  asm ("pushw  %%es\n\t"
       "movw   %%ds, %%ax\n\t"
       "movw   %%ax, %%es\n\t"
       "movw   $0x1684, %%ax\n\t"
       "int    $0x2f\n\t"
       "movw   %%es, %%ax\n\t"
       "popw   %%es"
       : "=a" (api->segment), "=D" (api->offset) : "b" (id), "D" (vxdname));
  api->offset &= 0xffff;
  return api->offset != 0;
}

#define VXDLDR_DEVICE_ID 0x0027

struct far_ptr vxdldr_api = {0, 0};

int open_vxdldr(void)
{
  if (!get_vxd_pm_api(VXDLDR_DEVICE_ID, NULL, &vxdldr_api)) {
    printf ("cannot open VXDLDR.\n");
    return 0;
  }

  printf ("VXDLDR api: seg %04x, offset %08x\n", vxdldr_api.segment,
	  vxdldr_api.offset);
  return 1;
}

#define VXDLDR_APIFUNC_GETVERSION (unsigned short) 0
#define VXDLDR_APIFUNC_LOADDEVICE (unsigned short) 1
#define VXDLDR_APIFUNC_UNLOADDEVICE (unsigned short) 2


int vxdldr_get_version(int *vers)
{
  int res;

  if (!vxdldr_api.offset)
    return 0;

  asm ("lcall  %2\n\t"
       "jnc    1f\n\t"
       "xor    %%ecx, %%ecx\n\t"
       "jmp    2f\n"
       "1:\tmov   $1, %%ecx\n"
       "2:"
       : "=a" (*vers), "=c" (res)
       : "m" (vxdldr_api), "a" (VXDLDR_APIFUNC_GETVERSION)
       : "ebx", "edx");
  return res;
}

int vxdldr_load_device(char *filename)
{
  int res;

  if (!vxdldr_api.offset)
    return 0;

  asm ("lcall  %1\n\t"
       "jc     1f\n\t"
       "xor    %%eax, %%eax\n\t"
       "1:"
       : "=a" (res)
       : "m" (vxdldr_api), "a" (VXDLDR_APIFUNC_LOADDEVICE), "d" (filename)
       : "ebx", "ecx", "edx", "esi", "edi");
  return res;
}

int vxdldr_unload_device(char *devicename)
{
  int res;

  if (!vxdldr_api.offset)
    return 0;

  asm ("lcall  %1\n\t"
       "jc     1f\n\t"
       "xor    %%eax, %%eax\n\t"
       "1:"
       : "=a" (res)
       : "m" (vxdldr_api), "a" (VXDLDR_APIFUNC_UNLOADDEVICE), 
         "d" (devicename), "b" (0)
       : "ebx", "ecx", "edx", "esi", "edi");
  return res;
}

char sockvxd_filename[] = "SOCK.VXD";
char sockvxd_devicename[] = "SOCK    ";
char sockvxd_devicenamenopad[] = "SOCK";

struct far_ptr sock_api = {0, 0};

static void
initialize(void)
{
  int version;
  int res;

  if (get_vxd_pm_api(0, sockvxd_devicename, &sock_api)) {
      /* printf ("SOCK is there!  seg %04x offset %08x\n", sock_api.segment,
	    sock_api.offset); */
    return;
  } else {
      /* printf ("SOCK not there yet!\n"); */
  }

  if (!open_vxdldr()) {
    printf ("error opening VXDLDR\n");
    exit (1);
  }

  if (!vxdldr_get_version(&version)) {
    printf ("error getting VXDLDR version\n");
    exit (1);
  }
  /* printf ("VXDLDR version %08x\n", version); */

  if (sock_api.offset == 0) {
    /* try to load */

    res = vxdldr_load_device(sockvxd_filename);
    if (res) {
	printf ("ERROR: VxdLDR returned LOAD result %d for SOCK.VXD\n", res); 
	exit (1);
    }

    if (get_vxd_pm_api(0, sockvxd_devicename, &sock_api)) {
	/* printf ("SOCK is there!  seg %04x offset %08x\n", sock_api.segment,
	      sock_api.offset); */
    } else {
	printf ("ERROR: SOCK still not there even after apparently loading it!\n");
	exit (1);
    }
  }
}
  

int sock_socket(int proto, int sock)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %1"
       : "=a" (res)
       : "m" (sock_api), "a" (1), "b" (proto), "D" (sock));
  return res;
}

int sock_close(int sock)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %1"
       : "=a" (res)
       : "m" (sock_api), "a" (2), "D" (sock));
  return res;
}

int sock_bind(int sock, unsigned short port, unsigned long ip)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %1"
       : "=a" (res)
       : "m" (sock_api), "a" (3), "D" (sock),
         "b" (ip), "d" (port));
  return res;
}


int sock_sendto(int sock, unsigned short port, unsigned long ip,
		char *buf, unsigned long length)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %1"
       : "=a" (res)
       : "m" (sock_api), "a" (4), "D" (sock),
         "b" (ip), "d" (port),
         "c" (length), "S" (buf));
  return res;
}

int sock_send(int sock, char *buf, unsigned long *length)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %2"
       : "=a" (res), "=c" (*length)
       : "m" (sock_api), "a" (8), "D" (sock),
         "c" (*length), "S" (buf));
  return res;
}

int sock_recvfrom(int sock, unsigned short *port, unsigned long *ip,
		  char *buf, unsigned long *length)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %4"
       : "=a" (res), "=c" (*length), "=b" (*ip), "=d" (*port)
       : "m" (sock_api), "a" (5), "D" (sock), 
         "c" (*length), "S" (buf));
  return res;
}

int sock_receive(int sock, char *buf, unsigned long length)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %1"
       : "=a" (res)
       : "m" (sock_api), "a" (9), "D" (sock), 
         "c" (length), "S" (buf));
  return res;
}
int sock_select(unsigned long *rfd, 
		unsigned long *wfd,
		unsigned long *efd,
		unsigned long *timeo)
{
  int res;
  unsigned long foo[3];
  if (!sock_api.offset)
    initialize();
  foo[0] = rfd ? *rfd : 0;
  foo[1] = wfd ? *wfd : 0;
  foo[2] = efd ? *efd : 0;
  asm volatile ("movl (%%esi), %%ebx\n\t"
		"movl 4(%%esi), %%ecx\n\t"
		"movl 8(%%esi), %%edx\n\t"
		"pushl %%esi\n\t"
		"lcall %2\n\t"
		"popl  %%esi\n\t"
		"movl  %%ebx, (%%esi)\n\t"
		"movl  %%ecx, 4(%%esi)\n\t"
		"movl  %%edx, 8(%%esi)"
		: "=a" (res), "=D" (*timeo)
		: "m" (sock_api), "a" (6), "S" (&foo[0]),
		  "D" (*timeo)
		: "ebx", "ecx", "edx");
  if (rfd) *rfd = foo[0];
  if (wfd) *wfd = foo[1];
  if (efd) *efd = foo[2];
  return res;
}

int sock_connect(int sock, unsigned short port, unsigned long ip)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %1"
       : "=a" (res)
       : "m" (sock_api), "a" (7), "D" (sock),
         "b" (ip), "d" (port));
  return res;
}

int sock_listen(int sock, int numcon)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %1"
       : "=a" (res)
       : "m" (sock_api), "a" (10), "D" (sock), "c" (numcon));
  return res;
}
  
int sock_stats(int sock, unsigned long *res2)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %2"
       : "=a" (res), "=b" (*res2)
       : "m" (sock_api), "a" (11), "D" (sock));
  return res;
}
  
int sock_accept(int sock, int sock2, unsigned short *port, unsigned long *ip)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %3"
       : "=a" (res), "=b" (*port), "=c" (*ip)
       : "m" (sock_api), "a" (12), "D" (sock), "d" (sock2));
  return res;
}

int sock_getname(int sock, unsigned short *port, unsigned long *ip)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %3"
       : "=a" (res), "=b" (*port), "=c" (*ip)
       : "m" (sock_api), "a" (13), "D" (sock));
  return res;
}

int sock_getpeer(int sock, unsigned short *port, unsigned long *ip)
{
  int res;
  if (!sock_api.offset)
    initialize();
  asm ("lcall %3"
       : "=a" (res), "=b" (*port), "=c" (*ip)
       : "m" (sock_api), "a" (14), "D" (sock));
  return res;
}


int sock_getsetmode(int sock, int subcmd, int *value)
{
    int res;

    if (!sock_api.offset)
	initialize();
    asm ("lcall %2"
	 : "=a" (res), "=d" (*value)
	 : "m" (sock_api), "a" (15), "D" (sock),
	   "c" (subcmd), "d" (*value));
    return res;
}
